"use strict";

const {
  execSync
} = require("child_process");
const {
  getClonedDirPath,
  getDeleteDirPath
} = require('./HelperFunctions/helpers');
const {
  getRepoName
} = require("../GitActions/gitActions");
const {
  removeFolder
} = require("../FileAndFolderOperations/removeFolder");
const {
  executeSynchronizedCommands
} = require('../General/executeSyncCommands');
const {
  type,
  endPoint,
  branch,
  cacheDirectory,
  name
} = require('../../../jsonUtils/commonLinterRepoDetails');
/**
 * @function cloneViaCdt - Using the "clint development tool" clones the common linter_configuration repo into precommit library
 * @returns {boolean} - indicating the success or failure of the cloning process
 */

function cloneViaCdt() {
  removeFolder(getDeleteDirPath());
  const commandToCloneCommonConfigRepo = `npx cdt clone --clone:type=${type} --clone:url=${endPoint} --clone:branch=${branch} --clone:cacheDir=${cacheDirectory} --clone:proj:name=${name}`;
  let isCommonConfigurationClonedSuccessfully = executeSynchronizedCommands(execSync, [commandToCloneCommonConfigRepo, {
    cwd: getClonedDirPath()
  }], `Lint Configuration Cloned Successfully - ${getRepoName() || 'common'}`, 'Could not clone the linters common repo', false, true);
  return isCommonConfigurationClonedSuccessfully;
}
module.exports = {
  cloneViaCdt
};