"use strict";

const {
  Logger
} = require('../Logger/Logger');

/**
 * @function executeMethodsThatReturnBooleanValue - wrapper method to execute certain commands/methods that return a boolean value
 * @param {string} consoleMessage - message that must be printed on failure of the method to be executed
 * @param {Function} _callback - the function to be executed
 * @param  {...any} params - arguments to be passed to the method to be executed
 */
async function executeMethodsThatReturnBooleanValue(consoleMessage, _callback, ...params) {
  if (!(await _callback(...params))) {
    throw new Error(consoleMessage);
  }
}

/**
 * 
 * @param {any} handleObject - default value to be retunred if the function to be executed encounters an exception
 * @param {string} consoleMessage - message to be logged if the function to be executed fails
 * @param {Function} _callback - method/command to be executed
 * @param  {...any} params - arguments to be passed to the method/command to be executed
 * @returns {any} - the result returned by the executed method/command or the default return value provided in argument
 */
function executeMethodsThatMayThrowException(handleObject, consoleMessage, _callback, ...params) {
  try {
    return _callback(...params);
  } catch (err) {
    Logger.log(Logger.FAILURE_TYPE, err);
    Logger.log(Logger.FAILURE_TYPE, consoleMessage);
    return handleObject;
  }
}
module.exports = {
  executeMethodsThatReturnBooleanValue,
  executeMethodsThatMayThrowException
};