"use strict";

const {
  readdirSync,
  existsSync
} = require('fs');
const path = require('path');
const {
  getNodeModulesPath
} = require('../General/getNodeModulesPath');
const {
  Logger
} = require('../Logger/Logger');
const {
  getLibraryInstalledLocation
} = require('../General/getLibraryInstalledLocation');
const {
  executeSynchronizedCommands
} = require('../General/executeSyncCommands');
const {
  repoURL
} = require('../../../jsonUtils/commonLinterRepoDetails');
const {
  executeMethodsThatMayThrowException
} = require('../General/wrapperFunctionToExecuteAFunction');
const {
  getServicePathElseCommon
} = require('../EslintConfigFileUtils/getLintConfiguration');
function getUnInstalledPlugins(pluginsForCurrentRepo, pluginNamesOfDevDependencyPlugins, devDependencies) {
  let pluginsInNodeModules = executeSynchronizedCommands(readdirSync, [path.join(getNodeModulesPath(), 'node_modules')], '', 'Unable to get the plugins inside node_modules', true, false);
  let pluginsToBeInstalled = [];
  pluginsForCurrentRepo.filter(plugin => {
    if (plugin.packageName.startsWith('@')) {
      let scope = plugin.packageName.split('/')[0];
      let pluginName = plugin.packageName.split('/')[1];
      let scopedPlugins = executeSynchronizedCommands(readdirSync, [path.join(getNodeModulesPath(), 'node_modules', scope)], '', 'Unable to get the plugins inside the scope inside node_modules', true, false);
      let isPluginInstalled = scopedPlugins.includes(pluginName) ? true : false;
      if (!isPluginInstalled) {
        pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`);
      }
    } else {
      let isPluginInstalled = pluginsInNodeModules.includes(plugin.packageName) ? true : false;
      if (!isPluginInstalled) {
        pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`);
      }
    }
    if (pluginNamesOfDevDependencyPlugins.includes(plugin.packageName)) {
      if (plugin.version !== devDependencies[plugin.packageName]) {
        pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`);
      }
    }
  });
  return pluginsToBeInstalled;
}
function getAllPlugins() {
  let serviceSpecificPlugins = [];
  const pathToCommonPluginsFile = path.join(getLibraryInstalledLocation(), 'configuration', 'common', 'pluginVersion.js');
  let commonPlugins = executeMethodsThatMayThrowException([], 'Make sure there is a pluginVersion.js file in common folder under configuration folder', require, pathToCommonPluginsFile);
  const {
    pluginVersionPath: serviceSpecificPluginsFile
  } = getServicePathElseCommon();
  if (serviceSpecificPluginsFile !== undefined) {
    serviceSpecificPlugins = executeMethodsThatMayThrowException([], `Kindly Ensure that Configuration is setup in Configuration repo \n ${repoURL}`, require, serviceSpecificPluginsFile);
  } else {
    serviceSpecificPlugins = [];
  }
  return [...commonPlugins, ...serviceSpecificPlugins];
}

/**
 * @function checkIfPluginsAreInstalled - to check if all plugins present in the common linter_configuration repository has been installed or not
 * @returns {Object} - contains two properties namely a string which says no plugins present for the repository or an empty string and an array containing plugins to be installed
 */
function checkIfPluginsAreInstalled() {
  const pluginsForCurrentRepo = getAllPlugins();
  let uninstalledPlugins = [];
  let plugin = {
    noPluginMessage: '',
    uninstalledPlugins: []
  };
  let pluginsIndevDependencies = {};
  let pluginNamesOfDevDependencyPlugins = [];
  let arePluginsPresentIndevDependencies = false;
  let arePluginsPresentForCurrentRepo = false;
  if (existsSync(path.join(getNodeModulesPath(), 'package.json'))) {
    let packageJsonContent = require(path.join(getNodeModulesPath(), 'package.json'));
    pluginsIndevDependencies = packageJsonContent.devDependencies;
    if (pluginsIndevDependencies !== undefined) {
      arePluginsPresentIndevDependencies = true;
      pluginNamesOfDevDependencyPlugins = Object.keys(pluginsIndevDependencies);
    } else if (pluginsIndevDependencies === undefined) {
      Logger.log(Logger.INFO_TYPE, 'No devDependencies present!');
    }
  } else {
    arePluginsPresentIndevDependencies = false;
  }
  arePluginsPresentForCurrentRepo = pluginsForCurrentRepo.length !== 0 ? true : false;
  if (arePluginsPresentForCurrentRepo || arePluginsPresentIndevDependencies) {
    uninstalledPlugins = getUnInstalledPlugins(pluginsForCurrentRepo, pluginNamesOfDevDependencyPlugins, pluginsIndevDependencies);
    plugin.noPluginMessage = '';
    plugin.uninstalledPlugins = uninstalledPlugins;
    return plugin;
  } else {
    plugin.noPluginMessage = 'No plugins present for the repository';
    plugin.uninstalledPlugins = [];
    return plugin;
  }
}
module.exports = {
  checkIfPluginsAreInstalled
};